@@ -59,7 +59,11 @@ func (clusterRequest *ClusterLoggingRequest) CreateOrUpdateCollection(proxyConfi
 			return
 		}
 		logger.Debugf("Generated collector config: %s", collectorConfig)
-		collectorConfHash = utils.CalculateMD5Hash(collectorConfig)
+		collectorConfHash, err = utils.CalculateMD5Hash(collectorConfig)
+		if err != nil {
+			logger.Errorf("unable to calculate MD5 hash. E: %s", err.Error())
+			return
+		}
 		if err = clusterRequest.createOrUpdateFluentdService(); err != nil {
 			return
 		}
@@ -84,26 +88,7 @@ func (clusterRequest *ClusterLoggingRequest) CreateOrUpdateCollection(proxyConfi
 			return
 		}
 
-		fluentdStatus, err := clusterRequest.getFluentdCollectorStatus()
-		if err != nil {
-			return fmt.Errorf("Failed to get status of Fluentd: %v", err)
-		}
-
-		printUpdateMessage := true
-		retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
-			if !compareFluentdCollectorStatus(fluentdStatus, cluster.Status.Collection.Logs.FluentdStatus) {
-				if printUpdateMessage {
-					logrus.Info("Updating status of Fluentd")
-					printUpdateMessage = false
-				}
-				cluster.Status.Collection.Logs.FluentdStatus = fluentdStatus
-				return clusterRequest.UpdateStatus(cluster)
-			}
-			return nil
-		})
-		if retryErr != nil {
-			return fmt.Errorf("Failed to update Cluster Logging Fluentd status: %v", retryErr)
-		}
+		clusterRequest.UpdateFluentdStatus()
 
 		if collectorServiceAccount != nil {
 
@@ -123,7 +108,10 @@ func (clusterRequest *ClusterLoggingRequest) CreateOrUpdateCollection(proxyConfi
 			return
 		}
 
-		clusterRequest.removeFluentd()
+		if err = clusterRequest.removeFluentd(); err != nil {
+			return
+		}
+
 		if err = clusterRequest.RemoveServiceAccount("logcollector"); err != nil {
 			return
 		}
@@ -136,6 +124,34 @@ func (clusterRequest *ClusterLoggingRequest) CreateOrUpdateCollection(proxyConfi
 	return nil
 }
 
+func (clusterRequest *ClusterLoggingRequest) UpdateFluentdStatus() (err error) {
+
+	cluster := clusterRequest.cluster
+
+	fluentdStatus, err := clusterRequest.getFluentdCollectorStatus()
+	if err != nil {
+		return fmt.Errorf("Failed to get status of Fluentd: %v", err)
+	}
+
+	printUpdateMessage := true
+	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
+		if !compareFluentdCollectorStatus(fluentdStatus, cluster.Status.Collection.Logs.FluentdStatus) {
+			if printUpdateMessage {
+				logrus.Info("Updating status of Fluentd")
+				printUpdateMessage = false
+			}
+			cluster.Status.Collection.Logs.FluentdStatus = fluentdStatus
+			return clusterRequest.UpdateStatus(cluster)
+		}
+		return nil
+	})
+	if retryErr != nil {
+		return fmt.Errorf("Failed to update Cluster Logging Fluentd status: %v", retryErr)
+	}
+
+	return nil
+}
+
 func compareFluentdCollectorStatus(lhs, rhs logging.FluentdCollectorStatus) bool {
 	if lhs.DaemonSet != rhs.DaemonSet {
 		return false
